@@ -33,12 +33,13 @@
 #include <glib/gi18n-lib.h>
 
 #include <libebook/e-contact.h>
+#include <libedataserver/e-data-server-util.h>
 #include <camel/camel.h>
 
-#include <em-operation-queue.h>
+#include <e-mapi-operation-queue.h>
 
-#include "exchange-mapi-utils.h"
-#include "exchange-mapi-defs.h"
+#include "e-mapi-utils.h"
+#include "e-mapi-defs.h"
 
 #include "e-book-backend-mapi.h"
 
@@ -46,10 +47,10 @@ G_DEFINE_TYPE (EBookBackendMAPI, e_book_backend_mapi, E_TYPE_BOOK_BACKEND)
 
 struct _EBookBackendMAPIPrivate
 {
-	EMOperationQueue *op_queue;
+	EMapiOperationQueue *op_queue;
 
 	GMutex *conn_lock;
-	ExchangeMapiConnection *conn;
+	EMapiConnection *conn;
 	gchar *profile;
 	gchar *book_uri;
 	gboolean marked_for_offline;
@@ -171,7 +172,7 @@ ebbm_set_cache_time (EBookBackendMAPI *ebma, glong cache_seconds)
 	g_free (iso_time);
 
 	if (error) {
-		exchange_mapi_debug_print ("%s: Failed to set value: %s", G_STRFUNC, error->message);
+		e_mapi_debug_print ("%s: Failed to set value: %s", G_STRFUNC, error->message);
 		g_error_free (error);
 	}
 }
@@ -234,7 +235,7 @@ ebbm_notify_connection_status (EBookBackendMAPI *ebma, gboolean is_online)
 }
 
 static void
-ebbm_fetch_contacts (EBookBackendMAPI *ebma, struct mapi_SRestriction *restriction, EDataBookView *book_view, glong *last_modification_secs, GError **error)
+ebbm_fetch_contacts (EBookBackendMAPI *ebma, BuildRestrictionsCB build_rs_cb, gpointer build_rs_cb_data, EDataBookView *book_view, glong *last_modification_secs, GError **error)
 {
 	EBookBackendMAPIClass *ebmac;
 	struct FetchContactsData notify_data = { 0 };
@@ -247,50 +248,12 @@ ebbm_fetch_contacts (EBookBackendMAPI *ebma, struct mapi_SRestriction *restricti
 	g_return_if_fail (ebmac != NULL);
 	g_return_if_fail (ebmac->op_fetch_contacts != NULL);
 
-	ebmac->op_fetch_contacts (ebma, restriction, book_view, &notify_data, error);
+	ebmac->op_fetch_contacts (ebma, build_rs_cb, build_rs_cb_data, book_view, &notify_data, error);
 
 	if (last_modification_secs && *last_modification_secs < notify_data.last_modification)
 		*last_modification_secs = notify_data.last_modification;
 }
 
-static struct mapi_SRestriction *
-ebbm_build_cache_update_restriction (EBookBackendMAPI *ebma, TALLOC_CTX *mem_ctx)
-{
-	struct mapi_SRestriction *restriction;
-	EBookBackendMAPIPrivate *priv;
-	struct SPropValue sprop;
-	struct timeval t = { 0 };
-	glong last_update_secs = 0;
-
-	g_return_val_if_fail (ebma != NULL, NULL);
-	g_return_val_if_fail (mem_ctx != NULL, NULL);
-	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), NULL);
-
-	priv = ebma->priv;
-	g_return_val_if_fail (priv != NULL, NULL);
-	g_return_val_if_fail (priv->db != NULL, NULL);
-
-	if (!ebbm_get_cache_time (ebma, &last_update_secs) || last_update_secs <= 0)
-		return NULL;
-
-	restriction = talloc_zero (mem_ctx, struct mapi_SRestriction);
-	g_assert (restriction != NULL);
-
-	restriction->rt = RES_PROPERTY;
-	restriction->res.resProperty.relop = RELOP_GE;
-	restriction->res.resProperty.ulPropTag = PR_LAST_MODIFICATION_TIME;
-
-	t.tv_sec = last_update_secs;
-	t.tv_usec = 0;
-
-	set_SPropValue_proptag_date_timeval (&sprop, PR_LAST_MODIFICATION_TIME, &t);
-
-	cast_mapi_SPropValue (mem_ctx, &(restriction->res.resProperty.lpProp),
-			      &sprop);
-
-	return restriction;
-}
-
 static gboolean
 unref_backend_idle_cb (gpointer data)
 {
@@ -303,6 +266,52 @@ unref_backend_idle_cb (gpointer data)
 	return FALSE;
 }
 
+static gboolean
+ebbm_build_cache_update_restriction (EMapiConnection *conn,
+				     mapi_id_t fid,
+				     TALLOC_CTX *mem_ctx,
+				     struct mapi_SRestriction **restrictions,
+				     gpointer user_data,
+				     GCancellable *cancellable,
+				     GError **perror)
+{
+	EBookBackendMAPI *ebma = user_data;
+	struct mapi_SRestriction *restriction;
+	EBookBackendMAPIPrivate *priv;
+	struct SPropValue sprop;
+	struct timeval t = { 0 };
+	glong last_update_secs = 0;
+
+	g_return_val_if_fail (ebma != NULL, FALSE);
+	g_return_val_if_fail (mem_ctx != NULL, FALSE);
+	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), FALSE);
+	g_return_val_if_fail (restrictions != NULL, FALSE);
+
+	priv = ebma->priv;
+	g_return_val_if_fail (priv != NULL, FALSE);
+	g_return_val_if_fail (priv->db != NULL, FALSE);
+
+	if (!ebbm_get_cache_time (ebma, &last_update_secs) || last_update_secs <= 0)
+		return TRUE;
+
+	restriction = talloc_zero (mem_ctx, struct mapi_SRestriction);
+	g_return_val_if_fail (restriction != NULL, FALSE);
+
+	restriction->rt = RES_PROPERTY;
+	restriction->res.resProperty.relop = RELOP_GE;
+	restriction->res.resProperty.ulPropTag = PR_LAST_MODIFICATION_TIME;
+
+	t.tv_sec = last_update_secs;
+	t.tv_usec = 0;
+
+	set_SPropValue_proptag_date_timeval (&sprop, PR_LAST_MODIFICATION_TIME, &t);
+	cast_mapi_SPropValue (mem_ctx, &(restriction->res.resProperty.lpProp), &sprop);
+
+	*restrictions = restriction;
+
+	return TRUE;
+}
+
 static gpointer
 ebbm_update_cache_cb (gpointer data)
 {
@@ -326,20 +335,12 @@ ebbm_update_cache_cb (gpointer data)
 	g_cancellable_reset (priv->update_cache);
 
 	if (!g_cancellable_is_cancelled (priv->update_cache) && ebmac->op_fetch_contacts) {
-		TALLOC_CTX *mem_ctx;
-		struct mapi_SRestriction *restriction;
-
-		mem_ctx = talloc_init (G_STRFUNC);
-		restriction = ebbm_build_cache_update_restriction (ebma, mem_ctx);
-
 		/* get time stored in a cache, to always use the latest last modification time */
 		if (!ebbm_get_cache_time (ebma, &last_modification_secs))
 			last_modification_secs = 0;
 
-		ebbm_fetch_contacts (ebma, restriction, NULL, &last_modification_secs, &error);
+		ebbm_fetch_contacts (ebma, ebbm_build_cache_update_restriction, ebma, NULL, &last_modification_secs, &error);
 		e_book_backend_sqlitedb_set_is_populated (priv->db, EMA_EBB_CACHE_FOLDERID, error != NULL, NULL);
-
-		talloc_free (mem_ctx);
 	}
 
 	if (!error && !g_cancellable_is_cancelled (priv->update_cache) && ebmac->op_fetch_known_uids) {
@@ -389,9 +390,9 @@ ebbm_connect_user (EBookBackendMAPI *ebma, GCancellable *cancellable, const gcha
 {
 	EBookBackendMAPIPrivate *priv = ebma->priv;
 	GError *mapi_error = NULL;
-	ExchangeMapiConnection *old_conn;
+	EMapiConnection *old_conn;
 
-	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma))) {
+	if (!e_backend_get_online (E_BACKEND (ebma))) {
 		ebbm_notify_connection_status (ebma, FALSE);
 	} else {
 		if (priv->update_cache_thread) {
@@ -410,13 +411,14 @@ ebbm_connect_user (EBookBackendMAPI *ebma, GCancellable *cancellable, const gcha
 		old_conn = priv->conn;
 		priv->conn = NULL;
 
-		priv->conn = exchange_mapi_connection_new (priv->profile,
+		priv->conn = e_mapi_connection_new (priv->profile,
 							   password,
+							   cancellable,
 							   &mapi_error);
 		if (!priv->conn) {
-			priv->conn = exchange_mapi_connection_find (priv->profile);
-			if (priv->conn && !exchange_mapi_connection_connected (priv->conn))
-				exchange_mapi_connection_reconnect (priv->conn, password, &mapi_error);
+			priv->conn = e_mapi_connection_find (priv->profile);
+			if (priv->conn && !e_mapi_connection_connected (priv->conn))
+				e_mapi_connection_reconnect (priv->conn, password, cancellable, &mapi_error);
 		}
 
 		if (old_conn)
@@ -456,7 +458,7 @@ static void
 ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_exists, GError **perror)
 {
 	EBookBackendMAPIPrivate *priv = ebma->priv;
-	ESource *source = e_book_backend_get_source (E_BOOK_BACKEND (ebma));
+	ESource *source = e_backend_get_source (E_BACKEND (ebma));
 	const gchar *offline;
 	const gchar *cache_dir, *krb_sso;
 	GError *error = NULL;
@@ -493,10 +495,10 @@ ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_e
 
 	e_book_backend_notify_readonly (E_BOOK_BACKEND (ebma), TRUE);
 
-	ebbm_notify_connection_status (ebma, e_book_backend_is_online (E_BOOK_BACKEND (ebma)));
+	ebbm_notify_connection_status (ebma, e_backend_get_online (E_BACKEND (ebma)));
 
 	/* Either we are in Online mode or this is marked for offline */
-	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma)) &&
+	if (!e_backend_get_online (E_BACKEND (ebma)) &&
 	    !priv->marked_for_offline) {
 		g_propagate_error (perror, EDB_ERROR (OFFLINE_UNAVAILABLE));
 		e_book_backend_notify_opened (E_BOOK_BACKEND (ebma), EDB_ERROR (OFFLINE_UNAVAILABLE));
@@ -504,7 +506,7 @@ ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_e
 	}
 
 	/* Once aunthentication in address book works this can be removed */
-	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma))) {
+	if (!e_backend_get_online (E_BACKEND (ebma))) {
 		e_book_backend_notify_online (E_BOOK_BACKEND (ebma), FALSE);
 		e_book_backend_notify_opened (E_BOOK_BACKEND (ebma), NULL /* Success */);
 		return;
@@ -597,7 +599,7 @@ ebbm_authenticate_user (EBookBackendMAPI *ebma, GCancellable *cancellable, ECred
 {
 	const gchar *password;
 
-	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma))) {
+	if (!e_backend_get_online (E_BACKEND (ebma))) {
 		ebbm_notify_connection_status (ebma, FALSE);
 	} else {
 		password = e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD);
@@ -606,19 +608,22 @@ ebbm_authenticate_user (EBookBackendMAPI *ebma, GCancellable *cancellable, ECred
 }
 
 static void
-ebbm_set_online (EBookBackend *backend, gboolean is_online)
+ebbm_notify_online_cb (EBookBackend *backend, GParamSpec *pspec)
 {
 	EBookBackendMAPI *ebma = E_BOOK_BACKEND_MAPI (backend);
 	EBookBackendMAPIPrivate *priv = ebma->priv;
 	ESource *esource;
 	const gchar *krb_sso = NULL;
+	gboolean online;
 
-	e_book_backend_notify_online (backend, is_online);
+	online = e_backend_get_online (E_BACKEND (backend));
+
+	e_book_backend_notify_online (backend, online);
 	if (e_book_backend_is_opened (backend)) {
 		e_book_backend_mapi_lock_connection (ebma);
 
-		esource = e_book_backend_get_source (E_BOOK_BACKEND (ebma));
-		if (!is_online) {
+		esource = e_backend_get_source (E_BACKEND (ebma));
+		if (!online) {
 			e_book_backend_notify_readonly (backend, TRUE);
 			ebbm_notify_connection_status (ebma, FALSE);
 
@@ -746,7 +751,7 @@ ebbm_book_view_thread (gpointer data)
 			/* todo: create restriction based on the book_view */
 			if (!error) {
 				g_cancellable_reset (priv->update_cache);
-				ebbm_fetch_contacts (bvtd->ebma, NULL, bvtd->book_view, NULL, &error);
+				ebbm_fetch_contacts (bvtd->ebma, NULL, NULL, bvtd->book_view, NULL, &error);
 				g_cancellable_cancel (priv->update_cache);
 			}
 
@@ -780,9 +785,9 @@ typedef enum {
 	OP_OPEN,
 	OP_REMOVE,
 
-	OP_CREATE_CONTACT,
+	OP_CREATE_CONTACTS,
 	OP_REMOVE_CONTACTS,
-	OP_MODIFY_CONTACT,
+	OP_MODIFY_CONTACTS,
 	OP_GET_CONTACT,
 	OP_GET_CONTACT_LIST,
 	OP_START_BOOK_VIEW,
@@ -820,8 +825,8 @@ typedef struct {
 typedef struct {
 	OperationBase base;
 
-	GSList *id_list;
-} OperationIDList;
+	GSList *str_slist;
+} OperationStrSlist;
 
 typedef struct {
 	OperationBase base;
@@ -871,37 +876,40 @@ ebbm_operation_cb (OperationBase *op, gboolean cancelled, EBookBackend *backend)
 			e_data_book_respond_remove (op->book, op->opid, error);
 		}
 	} break;
-	case OP_CREATE_CONTACT: {
-		OperationStr *ops = (OperationStr *) op;
-		const gchar *vcard = ops->str;
+	case OP_CREATE_CONTACTS: {
+		OperationStrSlist *ops = (OperationStrSlist *) op;
 
 		if (!cancelled) {
-			EContact *contact = NULL;
+			GSList *added_contacts = NULL;
 
-			if (ebmac->op_create_contact)
-				ebmac->op_create_contact (ebma, op->cancellable, vcard, &contact, &error);
+			if (ebmac->op_create_contacts)
+				ebmac->op_create_contacts (ebma, op->cancellable, ops->str_slist, &added_contacts, &error);
 			else
 				error = EDB_ERROR (NOT_SUPPORTED);
 
-			if (contact && !error)
-				e_book_backend_mapi_notify_contact_update (ebma, NULL, contact, NULL, -1, -1, NULL);
+			if (added_contacts && !error) {
+				const GSList *l;
 
-			e_data_book_respond_create (op->book, op->opid, error, contact);
+				for (l = added_contacts; l; l = l->next) {
+					e_book_backend_mapi_notify_contact_update (ebma, NULL, E_CONTACT (l->data), NULL, -1, -1, NULL);
+				}
+			}
 
-			if (contact)
-				g_object_unref (contact);
+			e_data_book_respond_create_contacts (op->book, op->opid, error, added_contacts);
+
+			e_util_free_object_slist (added_contacts);
 		}
 
-		g_free (ops->str);
+		e_util_free_string_slist (ops->str_slist);
 	} break;
 	case OP_REMOVE_CONTACTS: {
-		OperationIDList *opil = (OperationIDList *) op;
+		OperationStrSlist *ops = (OperationStrSlist *) op;
 
 		if (!cancelled) {
 			GSList *removed_ids = NULL;
 
 			if (ebmac->op_remove_contacts)
-				ebmac->op_remove_contacts (ebma, op->cancellable, opil->id_list, &removed_ids, &error);
+				ebmac->op_remove_contacts (ebma, op->cancellable, ops->str_slist, &removed_ids, &error);
 			else
 				error = EDB_ERROR (NOT_SUPPORTED);
 
@@ -922,31 +930,33 @@ ebbm_operation_cb (OperationBase *op, gboolean cancelled, EBookBackend *backend)
 			g_slist_free (removed_ids);
 		}
 
-		g_slist_foreach (opil->id_list, (GFunc) g_free, NULL);
-		g_slist_free (opil->id_list);
+		e_util_free_string_slist (ops->str_slist);
 	} break;
-	case OP_MODIFY_CONTACT: {
-		OperationStr *ops = (OperationStr *) op;
-		const gchar *vcard = ops->str;
+	case OP_MODIFY_CONTACTS: {
+		OperationStrSlist *ops = (OperationStrSlist *) op;
 
 		if (!cancelled) {
-			EContact *contact = NULL;
+			GSList *modified_contacts = NULL;
 
-			if (ebmac->op_modify_contact)
-				ebmac->op_modify_contact (ebma, op->cancellable, vcard, &contact, &error);
+			if (ebmac->op_modify_contacts)
+				ebmac->op_modify_contacts (ebma, op->cancellable, ops->str_slist, &modified_contacts, &error);
 			else
 				error = EDB_ERROR (NOT_SUPPORTED);
 
-			if (contact && !error)
-				e_book_backend_mapi_notify_contact_update (ebma, NULL, contact, NULL, -1, -1, NULL);
+			if (modified_contacts && !error) {
+				const GSList *l;
 
-			e_data_book_respond_modify (op->book, op->opid, error, contact);
+				for (l = modified_contacts; l; l = l->next) {
+					e_book_backend_mapi_notify_contact_update (ebma, NULL, E_CONTACT (l->data), NULL, -1, -1, NULL);
+				}
+			}
 
-			if (contact)
-				g_object_unref (contact);
+			e_data_book_respond_modify_contacts (op->book, op->opid, error, modified_contacts);
+
+			e_util_free_object_slist (modified_contacts);
 		}
 
-		g_free (ops->str);
+		e_util_free_string_slist (ops->str_slist);
 	} break;
 	case OP_GET_CONTACT: {
 		OperationStr *ops = (OperationStr *) op;
@@ -1083,7 +1093,7 @@ base_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, GCancell
 	op->opid = opid;
 	op->cancellable = cancellable;
 
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 static void
@@ -1113,7 +1123,43 @@ str_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, GCancella
 	op->base.cancellable = cancellable;
 	op->str = g_strdup (str);
 
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
+}
+
+static void
+str_slist_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable *cancellable, const GSList *str_slist, OperationType ot)
+{
+	OperationStrSlist *op;
+	EBookBackendMAPI *ebbm;
+	EBookBackendMAPIPrivate *priv;
+	GSList *l;
+
+	g_return_if_fail (backend != NULL);
+	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (backend));
+	g_return_if_fail (str_slist != NULL);
+
+	ebbm = E_BOOK_BACKEND_MAPI (backend);
+	priv = ebbm->priv;
+	g_return_if_fail (priv != NULL);
+
+	g_object_ref (ebbm);
+	if (book)
+		g_object_ref (book);
+	if (cancellable)
+		g_object_ref (cancellable);
+
+	op = g_new0 (OperationStrSlist, 1);
+	op->base.ot = ot;
+	op->base.book = book;
+	op->base.opid = opid;
+	op->base.cancellable = cancellable;
+	op->str_slist = g_slist_copy ((GSList *) str_slist);
+
+	for (l = op->str_slist; l; l = l->next) {
+		l->data = g_strdup (l->data);
+	}
+
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 #define BASE_OP_DEF(_func, _ot)								\
@@ -1130,9 +1176,17 @@ _func (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable *cance
 	str_op_abstract (backend, book, opid, cancellable, str, _ot);		\
 }
 
+#define STR_SLIST_OP_DEF(_func, _ot)							\
+static void										\
+_func (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable *cancellable, const GSList *str_slist)	\
+{											\
+	str_slist_op_abstract (backend, book, opid, cancellable, str_slist, _ot);	\
+}
+
 BASE_OP_DEF (ebbm_op_remove, OP_REMOVE)
-STR_OP_DEF  (ebbm_op_create_contact, OP_CREATE_CONTACT)
-STR_OP_DEF  (ebbm_op_modify_contact, OP_MODIFY_CONTACT)
+STR_SLIST_OP_DEF (ebbm_op_create_contacts, OP_CREATE_CONTACTS)
+STR_SLIST_OP_DEF (ebbm_op_modify_contacts, OP_MODIFY_CONTACTS)
+STR_SLIST_OP_DEF (ebbm_op_remove_contacts, OP_REMOVE_CONTACTS)
 STR_OP_DEF  (ebbm_op_get_contact, OP_GET_CONTACT)
 STR_OP_DEF  (ebbm_op_get_contact_list, OP_GET_CONTACT_LIST)
 STR_OP_DEF  (ebbm_op_get_backend_property, OP_GET_BACKEND_PROPERTY)
@@ -1164,43 +1218,7 @@ ebbm_op_open (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable
 	op->base.cancellable = cancellable;
 	op->only_if_exists = only_if_exists;
 
-	em_operation_queue_push (priv->op_queue, op);
-}
-
-static void
-ebbm_op_remove_contacts (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable *cancellable, const GSList *id_list)
-{
-	OperationIDList *op;
-	EBookBackendMAPI *ebbm;
-	EBookBackendMAPIPrivate *priv;
-	GSList *l;
-
-	g_return_if_fail (backend != NULL);
-	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (backend));
-	g_return_if_fail (id_list != NULL);
-
-	ebbm = E_BOOK_BACKEND_MAPI (backend);
-	priv = ebbm->priv;
-	g_return_if_fail (priv != NULL);
-
-	g_object_ref (ebbm);
-	if (book)
-		g_object_ref (book);
-	if (cancellable)
-		g_object_ref (cancellable);
-
-	op = g_new0 (OperationIDList, 1);
-	op->base.ot = OP_REMOVE_CONTACTS;
-	op->base.book = book;
-	op->base.opid = opid;
-	op->base.cancellable = cancellable;
-	op->id_list = g_slist_copy ((GSList *) id_list);
-
-	for (l = op->id_list; l; l = l->next) {
-		l->data = g_strdup (l->data);
-	}
-
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 static void
@@ -1228,7 +1246,7 @@ ebbm_op_start_book_view (EBookBackend *backend, EDataBookView *book_view)
 
 	g_hash_table_insert (priv->running_book_views, book_view, GINT_TO_POINTER(1));
 
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 static void
@@ -1256,7 +1274,7 @@ ebbm_op_stop_book_view (EBookBackend *backend, EDataBookView *book_view)
 
 	g_hash_table_remove (priv->running_book_views, book_view);
 
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 static void
@@ -1285,7 +1303,7 @@ ebbm_op_authenticate_user (EBookBackend *backend, GCancellable *cancellable, ECr
 	op->base.cancellable = cancellable;
 	op->credentials = e_credentials_new_clone (credentials);
 
-	em_operation_queue_push (priv->op_queue, op);
+	e_mapi_operation_queue_push (priv->op_queue, op);
 }
 
 static void
@@ -1293,12 +1311,16 @@ e_book_backend_mapi_init (EBookBackendMAPI *ebma)
 {
 	ebma->priv = G_TYPE_INSTANCE_GET_PRIVATE (ebma, E_TYPE_BOOK_BACKEND_MAPI, EBookBackendMAPIPrivate);
 
-	ebma->priv->op_queue = em_operation_queue_new ((EMOperationQueueFunc) ebbm_operation_cb, ebma);
+	ebma->priv->op_queue = e_mapi_operation_queue_new ((EMapiOperationQueueFunc) ebbm_operation_cb, ebma);
 	ebma->priv->running_book_views = g_hash_table_new (g_direct_hash, g_direct_equal);
 	ebma->priv->conn_lock = g_mutex_new ();
 
 	ebma->priv->update_cache = g_cancellable_new ();
 	ebma->priv->update_cache_thread = NULL;
+
+	g_signal_connect (
+		ebma, "notify::online",
+		G_CALLBACK (ebbm_notify_online_cb), NULL);
 }
 
 static void
@@ -1353,17 +1375,15 @@ e_book_backend_mapi_class_init (EBookBackendMAPIClass *klass)
 
 	backend_class->open			  = ebbm_op_open;
 	backend_class->remove			  = ebbm_op_remove;
-	backend_class->create_contact		  = ebbm_op_create_contact;
+	backend_class->create_contacts		  = ebbm_op_create_contacts;
 	backend_class->remove_contacts		  = ebbm_op_remove_contacts;
-	backend_class->modify_contact		  = ebbm_op_modify_contact;
+	backend_class->modify_contacts		  = ebbm_op_modify_contacts;
 	backend_class->get_contact                = ebbm_op_get_contact;
 	backend_class->get_contact_list           = ebbm_op_get_contact_list;
 	backend_class->start_book_view            = ebbm_op_start_book_view;
 	backend_class->stop_book_view             = ebbm_op_stop_book_view;
 	backend_class->authenticate_user          = ebbm_op_authenticate_user;
 	backend_class->get_backend_property	  = ebbm_op_get_backend_property;
-	backend_class->set_online                 = ebbm_set_online;
-
 	klass->op_open				  = ebbm_open;
 	klass->op_remove                          = ebbm_remove;
 	klass->op_authenticate_user               = ebbm_authenticate_user;
@@ -1421,7 +1441,7 @@ e_book_backend_mapi_unlock_connection (EBookBackendMAPI *ebma)
 	g_mutex_unlock (ebma->priv->conn_lock);
 }
 
-ExchangeMapiConnection *
+EMapiConnection *
 e_book_backend_mapi_get_connection (EBookBackendMAPI *ebma)
 {
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), NULL);
@@ -1626,7 +1646,13 @@ e_book_backend_mapi_cache_get (EBookBackendMAPI *ebma, const gchar *key)
 
 /* 'data' is one of GET_ALL_KNOWN_IDS or GET_UIDS_ONLY */
 gboolean
-mapi_book_utils_get_prop_list (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropTagArray *props, gpointer data)
+mapi_book_utils_get_prop_list (EMapiConnection *conn,
+			       mapi_id_t fid,
+			       TALLOC_CTX *mem_ctx,
+			       struct SPropTagArray *props,
+			       gpointer data,
+			       GCancellable *cancellable,
+			       GError **perror)
 {
 	/* this is a list of all known book MAPI tag IDs;
 	   if you add new add it here too, otherwise it may not be fetched */
@@ -1707,16 +1733,16 @@ mapi_book_utils_get_prop_list (ExchangeMapiConnection *conn, mapi_id_t fid, TALL
 	g_return_val_if_fail (props != NULL, FALSE);
 
 	if (data == GET_UIDS_ONLY)
-		return exchange_mapi_utils_add_props_to_props_array (mem_ctx, props, uids_only_ids, G_N_ELEMENTS (uids_only_ids));
+		return e_mapi_utils_add_props_to_props_array (mem_ctx, props, uids_only_ids, G_N_ELEMENTS (uids_only_ids));
 
-	if (data == GET_ALL_KNOWN_IDS && !exchange_mapi_utils_add_props_to_props_array (mem_ctx, props, known_book_mapi_ids, G_N_ELEMENTS (known_book_mapi_ids)))
+	if (data == GET_ALL_KNOWN_IDS && !e_mapi_utils_add_props_to_props_array (mem_ctx, props, known_book_mapi_ids, G_N_ELEMENTS (known_book_mapi_ids)))
 		return FALSE;
 
 	/* called with fid = 0 from GAL */
 	if (!fid)
-		fid = exchange_mapi_connection_get_default_folder_id (conn, olFolderContacts, NULL);
+		fid = e_mapi_connection_get_default_folder_id (conn, olFolderContacts, cancellable, NULL);
 
-	return exchange_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, nids, G_N_ELEMENTS (nids));
+	return e_mapi_utils_add_named_ids_to_props_array (conn, fid, mem_ctx, props, nids, G_N_ELEMENTS (nids), cancellable, perror);
 }
 
 static gchar *
@@ -1744,7 +1770,7 @@ not_null (gconstpointer ptr)
 
 /* This is not setting E_CONTACT_UID */
 EContact *
-mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid, const gchar *book_uri, struct mapi_SPropValue_array *mapi_properties, struct SRow *aRow)
+mapi_book_utils_contact_from_props (EMapiConnection *conn, mapi_id_t fid, const gchar *book_uri, struct mapi_SPropValue_array *mapi_properties, struct SRow *aRow)
 {
 	EContact *contact = e_contact_new ();
 	gint i;
@@ -1752,9 +1778,9 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 	if (book_uri)
 		e_contact_set (contact, E_CONTACT_BOOK_URI, book_uri);
 
-	#define get_proptag(proptag) (aRow ? exchange_mapi_util_find_row_propval (aRow, proptag) : exchange_mapi_util_find_array_propval (mapi_properties, proptag))
+	#define get_proptag(proptag) (aRow ? e_mapi_util_find_row_propval (aRow, proptag) : e_mapi_util_find_array_propval (mapi_properties, proptag))
 	#define get_str_proptag(proptag) not_null (get_proptag (proptag))
-	#define get_namedid(nid) (aRow ? exchange_mapi_util_find_row_namedid (aRow, conn, fid, nid) : exchange_mapi_util_find_array_namedid (mapi_properties, conn, fid, nid))
+	#define get_namedid(nid) (aRow ? e_mapi_util_find_row_namedid (aRow, conn, fid, nid) : e_mapi_util_find_array_namedid (mapi_properties, conn, fid, nid))
 	#define get_str_namedid(nid) not_null (get_namedid (nid))
 
 	if (g_str_equal (get_str_proptag (PR_MESSAGE_CLASS), IPM_DISTLIST)) {
@@ -1785,7 +1811,7 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 
 			br.lpb = members->bin[i].lpb;
 			br.cb = members->bin[i].cb;
-			if (exchange_mapi_util_recip_entryid_decode (conn, &br, &display_name, &email)) {
+			if (e_mapi_util_recip_entryid_decode (conn, &br, &display_name, &email)) {
 				EVCardAttribute *attr;
 				gchar *value;
 				CamelInternetAddress *addr;
@@ -1855,7 +1881,7 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				gchar *buff = NULL;
 				GTimeVal tv;
 
-				tv.tv_sec = exchange_mapi_util_filetime_to_time_t (t);
+				tv.tv_sec = e_mapi_util_filetime_to_time_t (t);
 				tv.tv_usec = 0;
 				
 				buff = g_time_val_to_iso8601 (&tv);
@@ -1883,7 +1909,7 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				if (value) {
 					EContactDate date = {0};
 
-					time = exchange_mapi_util_filetime_to_time_t (t);
+					time = e_mapi_util_filetime_to_time_t (t);
 					tmtime = gmtime (&time);
 
 					date.day = tmtime->tm_mday;
